package com.alterdekim;

import com.alterdekim.db.Database;
import com.alterdekim.login.LoginServer;
import com.alterdekim.services.*;
import com.alterdekim.utils.BattleNetPacket;
import com.alterdekim.utils.Compute32;
import com.alterdekim.utils.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLSocket;
import java.io.*;
import java.util.HashMap;
import java.util.Map;

public class Connection extends Thread {

    /*
        All files were merged into protocol.proto
     */

    private static final Logger log = LoggerFactory.getLogger(Connection.class);

    private final SSLSocket fromClient;
    private OutputStream outToClient;

    private Map<Integer, Service> services;

    private final Database db;

    public Connection(SSLSocket socket, Database db) {
        this.fromClient = socket;
        this.db = db;
        this.init();
        this.start();
    }

    public void stopListeningAndDisconnect() {
        log.warn("Tried to stopListening");
    }

    public Map<Integer, Service> getServices() {
        return services;
    }

    public void send(BattleNetPacket bp) throws Exception {
       // log.info("Sending response: " + bp.getHeader().getServiceId() + " / " + bp.getHeader().getMethodId());
        this.outToClient.write(bp.Encode());
        this.outToClient.flush();
    }

    public void _send( byte[] b ) throws Exception {
        this.outToClient.write(b);
        this.outToClient.flush();
    }

    public Database getDb() {
        return db;
    }

    private void init() {
        this.services = new HashMap<>();
        this.services.put(0, new ConnectionService(""));
        this.services.put(1, new AuthService("bnet.protocol.authentication.AuthenticationClient")); // Compute32.Hash("bnet.protocol.authentication.AuthenticationServer")
        this.services.put(17, new AccountService("bnet.protocol.account.AccountNotify")); // Compute32.Hash("bnet.protocol.account.AccountService")
        this.services.put(4, new PresenceService("bnet.protocol.presence.PresenceService")); // Compute32.Hash("bnet.protocol.presence.PresenceService")
        this.services.put(6, new FriendsService("bnet.protocol.friends.FriendsNotify")); // Compute32.Hash("bnet.protocol.friends.FriendsService")
        this.services.put(7, new ChannelService("bnet.protocol.channel.Channel")); // Compute32.Hash("bnet.protocol.channel.Channel")
        this.services.put(16, new ResourcesService("bnet.protocol.resources.Resources"));
        this.services.put(9, new GameUtilitiesService("bnet.protocol.game_utilities.GameUtilities"));
    }

    private void processPacket( BattleNetPacket packet ) {
        try {

            Service is = this.services.get(Integer.parseUnsignedInt(Integer.toUnsignedString(packet.getHeader().getServiceId())));
            if( is == null ) {
                log.error("Can't process weird ServiceId: " + Integer.parseUnsignedInt(Integer.toUnsignedString(packet.getHeader().getServiceId())));
                return;
            }
            log.info("Incoming: " + is.getClass().getSimpleName() + " / " + packet.getHeader().getMethodId() + " (Token: " + packet.getHeader().getToken() + ")");
            is.process(packet, this);
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    @Override
    public void run() {
        try {
            InputStream is = fromClient.getInputStream();
            outToClient = fromClient.getOutputStream();
            while (true) {
                try {
                    int count;
                    byte[] buffer = new byte[4096]; // or 4096, or more
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    while ((count = is.read(buffer)) > 0) {
                        baos.write(buffer, 0, count);
                        byte[] r = baos.toByteArray();
                        BattleNetPacket bp = new BattleNetPacket();
                        int offset = 0;
                        while( offset < r.length ) {
                            offset += bp.Decode(r, offset, r.length);
                            processPacket(bp);
                        }
                        baos.reset();
                    }
                    baos.close();
                } catch (Exception e) {
                    e.printStackTrace();
                    break;
                }
            }
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        /*
                        br.close();
                fromClient.close();
         */
    }
}
