package com.alterdekim.services;

import com.alterdekim.Connection;
import com.alterdekim.Protocol;
import com.alterdekim.utils.BattleNetPacket;
import com.alterdekim.utils.Compute32;
import com.alterdekim.utils.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.nio.charset.StandardCharsets;

public class AuthService extends Service {

    private static final Logger log = LoggerFactory.getLogger(AuthService.class);

    public AuthService(String export_name) {
        super(export_name);
    }


    private void logonRequest( BattleNetPacket packet, Connection conn ) throws Exception {
        // Protocol.LogonRequest lr = Protocol.LogonRequest.parseFrom(packet.getBody());

        Protocol.Header h = Protocol.Header.newBuilder()
                .setServiceId(254)
                .setObjectId(0)
                .setToken(packet.getHeader().getToken())
                .setStatus(0)
                .setSize(0)
                .build();

        conn.send(new BattleNetPacket(h, new byte[0]));

        byte[] b = Util.hexStringToByteArray("080110C383A8F4CEBDCC021800");

        h = Protocol.Header.newBuilder()
                .setServiceId(1)
                .setObjectId(0)
                .setToken(0)
                .setMethodId(12)
                .setStatus(0)
                .setSize(b.length)
                .build();

        conn.send(new BattleNetPacket(h, b));

        h = Protocol.Header.newBuilder()
                .setServiceId(1)
                .setObjectId(0)
                .setToken(1)
                .setMethodId(13)
                .setStatus(0)
                .setSize(0)
                .build();

        conn.send(new BattleNetPacket(h, new byte[0]));
        //120C7765625F617574685F75726C1A16687474703A2F2F68732F6C6F67696E2F7A2E68746D6C
        byte[] us = "http://10.66.66.3:8080".getBytes(StandardCharsets.UTF_8);
        byte[] bb = Util.hexStringToByteArray("120C7765625F617574685F75726C1A"+Util.intToHex(us.length)+Util.bytesToHex(us));

        h = Protocol.Header.newBuilder()
                .setServiceId(8)
                .setObjectId(0)
                .setToken(2)
                .setMethodId(3)
                .setStatus(0)
                .setSize(bb.length)
                .build();

        conn.send(new BattleNetPacket(h, bb));
    }

    @Override
    public void process(BattleNetPacket packet, Connection conn) throws Exception {
        switch (packet.getHeader().getMethodId()) {
            case 1:
               logonRequest(packet, conn);
                break;
            case 4:
                Protocol.Header nnh = Protocol.Header.newBuilder()
                        .setStatus(0)
                        .setServiceId(254)
                        .setObjectId(0)
                        .setToken(packet.getHeader().getToken())
                        .setSize(0)
                        .build();

                conn.send(new BattleNetPacket(nnh, new byte[0]));
                break;
            case 7:
                Protocol.VerifyWebCredentialsRequest verifyWebCredentialsRequest
                        = Protocol.VerifyWebCredentialsRequest.parseFrom(packet.getBody());
                log.info( new String( verifyWebCredentialsRequest.getWebCredentials().toByteArray() ) );

                Protocol.Header h = Protocol.Header.newBuilder()
                        .setServiceId(254)
                        .setToken(packet.getHeader().getToken())
                        .setObjectId(0)
                        .setSize(0)
                        .setStatus(0)
                        .build();

                conn.send(new BattleNetPacket(h, new byte[0]));

                h = Protocol.Header.newBuilder()
                        .setServiceId(10)
                        .setMethodId(1)
                        .setToken(4)
                        .setObjectId(0)
                        .setSize(2)
                        .setStatus(0)
                        .build();

                conn.send(new BattleNetPacket(h, Util.hexStringToByteArray("0800")));

                h = Protocol.Header.newBuilder()
                        .setServiceId(10)
                        .setMethodId(1)
                        .setToken(5)
                        .setObjectId(0)
                        .setSize(2)
                        .setStatus(0)
                        .build();

                conn.send(new BattleNetPacket(h, Util.hexStringToByteArray("0800")));
                // THIRD
                //Protocol.LogonResult lr = Protocol.LogonResult.parseFrom(Util.hexStringToByteArray("080012120900000000000000011193710E1A000000001A12094743545702000002116739AB040000000022002801280228032806286230023A0F517569726B794F72632332393638384202534B4A40AA15061771938C7790CDA59BD500BF2AA3205912148098F8B68ADA802F1122B2792D90FA2952A338032A03895322DD02C4E2F3CFE6CD72415466B3798C9ADB015000"));

                Protocol.LogonResult logonResult = Protocol.LogonResult.newBuilder()
                                .setErrorCode(0)
                                .setAccount(Protocol.EntityId.newBuilder().setHigh(72057594037927936L).setLow(437154195L).build())
                                .addGameAccount(Protocol.EntityId.newBuilder().setHigh(144115198130930503L).setLow(78330215L).build())
                                .setEmail("")
                                .addAvailableRegion(1)
                                .addAvailableRegion(2)
                                .addAvailableRegion(3)
                                .addAvailableRegion(6)
                                .addAvailableRegion(98)
                                .setConnectedRegion(2)
                                .setBattleTag("Asmodeus#66666")
                                .setGeoipCountry("SK")
                                .build();

                h = Protocol.Header.newBuilder()
                        .setServiceId(1)
                        .setMethodId(5)
                        .setToken(6)
                        .setObjectId(0)
                        .setSize(logonResult.getSerializedSize())
                        .setStatus(0)
                        .build();

                conn.send(new BattleNetPacket(h, logonResult.toByteArray()));
                break;
            default:
                log.error("Can't process weird AuthService method: " + packet.getHeader().getMethodId());
                break;
        }
    }
}
